package clientpool

import (
	"context"
	"github.com/ethereum/go-ethereum/ethclient"
	"github.com/ethereum/go-ethereum/rpc"
	log "github.com/sirupsen/logrus"
	"net/http"
	"net/url"
	"sync"
	"time"
)

// use atomic pool

type poolConfig struct {
	RPCNode string
	Index   int32
	clients []*ethclient.Client
	mux     sync.Mutex
}

var pconfig = &poolConfig{}

func dialUrl(rpcurl string, proxy string) (*ethclient.Client, error) {
	if proxy == "" {
		return ethclient.Dial(rpcurl)
	} else {
		// set http proxy
		purl, err := url.Parse(proxy)
		if err != nil {
			return ethclient.Dial(rpcurl)
		}
		transport := http.DefaultTransport.(*http.Transport).Clone()
		transport.Proxy = http.ProxyURL(purl)
		hcli := &http.Client{
			Transport: transport,
			Timeout:   time.Second * 20,
		}
		rpcCli, err := rpc.DialOptions(context.Background(), rpcurl, rpc.WithHTTPClient(hcli))
		if err != nil {
			return ethclient.Dial(rpcurl)
		}
		log.WithField("proxy", proxy).Info("create eth client use proxy")
		return ethclient.NewClient(rpcCli), nil
	}
}

func InitPool(cnt int, rpc string, proxy string) {
	pconfig.Index = 0
	pconfig.RPCNode = rpc
	pconfig.clients = make([]*ethclient.Client, 0)
	for i := 0; i < cnt; i++ {
		if c, err := dialUrl(pconfig.RPCNode, proxy); err != nil {
			log.WithError(err).Fatal("failed to connect to rpc node")
		} else {
			pconfig.clients = append(pconfig.clients, c)
		}
	}
}

func (p *poolConfig) getClient() *ethclient.Client {
	p.mux.Lock()
	defer p.mux.Unlock()
	p.Index += 1
	idx := p.Index % int32(len(p.clients))
	c := p.clients[idx]
	return c
}

func GetClient() *ethclient.Client {
	return pconfig.getClient()
}
